/*
    KMid Backend using the ALSA Sequencer
    Copyright (C) 2009-2010 Pedro Lopez-Cabanillas <plcl@users.sf.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "alsabackend.h"
#include "alsamidiobject.h"
#include "alsamidioutput.h"

#include <kdemacros.h>
#include <KPluginFactory>
#include <KPluginLoader>
#include <KMessageBox>
#include <KLocale>

namespace KMid {

    class ALSABackend::BackendPrivate {
    public:
        BackendPrivate():
                      m_initialized(false),
                      m_backendString("ALSA Sequencer"),
                      m_object(NULL),
                      m_output(NULL)
        {}

        bool m_initialized;
        QString m_backendString;
        ALSAMIDIObject *m_object;
        ALSAMIDIOutput *m_output;
    };

    ALSABackend::ALSABackend(QObject* parent, const QVariantList& args)
        : Backend(parent, args), d(new BackendPrivate)
    {
        try {
            d->m_object = new ALSAMIDIObject(this);
            d->m_output = new ALSAMIDIOutput(this);
            d->m_object->initialize(d->m_output);
            d->m_initialized = true;
        } catch (SequencerError& ex) {
            QString errorstr = i18nc("@info","Fatal error from the ALSA sequencer backend. "
                "This usually happens when the kernel does not have ALSA support, "
                "the device node (/dev/snd/seq) does not exist, "
                "or the kernel module (snd_seq) is not loaded. "
                "Please check your ALSA/MIDI configuration. "
                "Returned error was: %1", ex.qstrError());
            KMessageBox::error(0, errorstr, i18nc("@title:window","ALSA Sequencer Backend Error"));
        }
    }

    ALSABackend::~ALSABackend()
    {
        delete d;
    }

    bool ALSABackend::initialized()
    {
        return d->m_initialized;
    }

    QString ALSABackend::backendName()
    {
        return d->m_backendString;
    }

    MIDIObject* ALSABackend::midiObject()
    {
        return d->m_object;
    }

    MIDIOutput* ALSABackend::midiOutput()
    {
        return d->m_output;
    }

    K_PLUGIN_FACTORY( ALSABackendFactory, registerPlugin<ALSABackend>(); )
    K_EXPORT_PLUGIN( ALSABackendFactory("kmid_alsa") )
}

